from chef import DataBag, DataBagItem, Search
from chef.exceptions import ChefError
from chef.tests import ChefTestCase


class DataBagTestCase(ChefTestCase):
    def test_list(self):
        bags = DataBag.list()
        self.assertIn("test_1", bags)
        self.assertIsInstance(bags["test_1"], DataBag)

    def test_keys(self):
        bag = DataBag("test_1")
        list(bag.keys()) == ["item_1", "item_2"]

    def test_item(self):
        bag = DataBag("test_1")
        item = bag["item_1"]
        self.assertEqual(item["test_attr"], 1)
        self.assertEqual(item["other"], "foo")

    def test_search_item(self):
        self.assertIn("test_1", Search.list())
        q = Search("test_1")
        self.assertIn("item_1", q)
        self.assertIn("item_2", q)
        self.assertEqual(q["item_1"]["raw_data"]["test_attr"], 1)
        item = q["item_1"].object
        self.assertIsInstance(item, DataBagItem)
        self.assertEqual(item["test_attr"], 1)

    def test_direct_item(self):
        item = DataBagItem("test_1", "item_1")
        self.assertEqual(item["test_attr"], 1)
        self.assertEqual(item["other"], "foo")

    def test_direct_item_bag(self):
        bag = DataBag("test_1")
        item = DataBagItem(bag, "item_1")
        self.assertEqual(item["test_attr"], 1)
        self.assertEqual(item["other"], "foo")

    def test_create_bag(self):
        name = self.random()
        bag = DataBag.create(name)
        self.register(bag)
        self.assertIn(name, DataBag.list())

    def test_create_item(self):
        value = self.random()
        bag_name = self.random()
        bag = DataBag.create(bag_name)
        self.register(bag)
        item_name = self.random()
        item = DataBagItem.create(bag, item_name, foo=value)
        self.assertIn("foo", item)
        self.assertEqual(item["foo"], value)
        self.assertIn(item_name, bag)
        bag2 = DataBag(bag_name)
        self.assertIn(item_name, bag2)
        item2 = bag2[item_name]
        self.assertIn("foo", item)
        self.assertEqual(item["foo"], value)

    def test_set_item(self):
        value = self.random()
        value2 = self.random()
        bag_name = self.random()
        bag = DataBag.create(bag_name)
        self.register(bag)
        item_name = self.random()
        item = DataBagItem.create(bag, item_name, foo=value)
        item["foo"] = value2
        item.save()
        self.assertEqual(item["foo"], value2)
        item2 = DataBagItem(bag, item_name)
        self.assertEqual(item2["foo"], value2)



from unittest import skip

from chef import Search, Node
from chef.exceptions import ChefError
from chef.tests import ChefTestCase, mockSearch


class SearchTestCase(ChefTestCase):
    def test_search_all(self):
        s = Search("node")
        self.assertGreaterEqual(len(s), 3)
        self.assertIn("test_1", s)
        self.assertIn("test_2", s)
        self.assertIn("test_3", s)

    def test_search_query(self):
        s = Search("node", "role:test_1")
        self.assertGreaterEqual(len(s), 2)
        self.assertIn("test_1", s)
        self.assertNotIn("test_2", s)
        self.assertIn("test_3", s)

    def test_list(self):
        searches = Search.list()
        self.assertIn("node", searches)
        self.assertIn("role", searches)

    def test_search_set_query(self):
        s = Search("node").query("role:test_1")
        self.assertGreaterEqual(len(s), 2)
        self.assertIn("test_1", s)
        self.assertNotIn("test_2", s)
        self.assertIn("test_3", s)

    def test_search_call(self):
        s = Search("node")("role:test_1")
        self.assertGreaterEqual(len(s), 2)
        self.assertIn("test_1", s)
        self.assertNotIn("test_2", s)
        self.assertIn("test_3", s)

    def test_rows(self):
        s = Search("node", rows=1)
        self.assertEqual(len(s), 1)
        self.assertGreaterEqual(s.total, 3)

    def test_start(self):
        s = Search("node", start=1)
        self.assertEqual(len(s), s.total - 1)
        self.assertGreaterEqual(s.total, 3)

    def test_slice(self):
        s = Search("node")[1:2]
        self.assertEqual(len(s), 1)
        self.assertGreaterEqual(s.total, 3)

        s2 = s[1:2]
        self.assertEqual(len(s2), 1)
        self.assertGreaterEqual(s2.total, 3)
        self.assertNotEqual(s[0]["name"], s2[0]["name"])

        s3 = Search("node")[2:3]
        self.assertEqual(len(s3), 1)
        self.assertGreaterEqual(s3.total, 3)
        self.assertEqual(s2[0]["name"], s3[0]["name"])

    def test_object(self):
        s = Search("node", "name:test_1")
        self.assertEqual(len(s), 1)
        node = s[0].object
        self.assertEqual(node.name, "test_1")
        self.assertEqual(node.run_list, ["role[test_1]"])


class MockSearchTestCase(ChefTestCase):
    @mockSearch({("node", "*:*"): [Node("fake_1", skip_load=True).to_dict()]})
    def test_single_node(self, MockSearch):
        import chef.search

        s = chef.search.Search("node")
        self.assertEqual(len(s), 1)
        self.assertIn("fake_1", s)



import six
from collections.abc import Mapping

import pkg_resources
from chef.acl import Acl

from chef.api import ChefAPI
from chef.exceptions import *


class ChefQuery(Mapping):
    def __init__(self, obj_class, names, api):
        self.obj_class = obj_class
        self.names = names
        self.api = api

    def __len__(self):
        return len(self.names)

    def __contains__(self, key):
        return key in self.names

    def __iter__(self):
        return iter(self.names)

    def __getitem__(self, name):
        if name not in self:
            raise KeyError("%s not found" % name)
        return self.obj_class(name, api=self.api)


class ChefObjectMeta(type):
    def __init__(cls, name, bases, d):
        super(ChefObjectMeta, cls).__init__(name, bases, d)
        if name != "ChefObject":
            ChefObject.types[name.lower()] = cls
        cls.api_version_parsed = pkg_resources.parse_version(cls.api_version)


class ChefObject(six.with_metaclass(ChefObjectMeta, object)):
    """A base class for Chef API objects."""

    types = {}

    url = ""
    attributes = {}

    api_version = "0.9"

    def __init__(self, name, api=None, skip_load=False):
        self.name = name
        self.api = api or ChefAPI.get_global()
        self._check_api_version(self.api)

        self.url = self.__class__.url + "/" + self.name
        self.exists = False
        data = {}
        if not skip_load:
            try:
                data = self.api[self.url]
            except ChefServerNotFoundError:
                pass
            else:
                self.exists = True
        self._populate(data)

    def _populate(self, data):
        for name, cls in six.iteritems(self.__class__.attributes):
            if name in data:
                value = cls(data[name])
            else:
                value = cls()
            setattr(self, name, value)

    @classmethod
    def from_search(cls, data, api=None):
        obj = cls(data.get("name"), api=api, skip_load=True)
        obj.exists = True
        obj._populate(data)
        return obj

    @classmethod
    def list(cls, api=None):
        """Return a :class:`ChefQuery` with the available objects of this type."""
        api = api or ChefAPI.get_global()
        cls._check_api_version(api)
        names = [name for name, url in six.iteritems(api[cls.url])]
        return ChefQuery(cls, names, api)

    @classmethod
    def create(cls, name, api=None, **kwargs):
        """Create a new object of this type. Pass the initial value for any
        attributes as keyword arguments.
        """
        api = api or ChefAPI.get_global()
        cls._check_api_version(api)
        obj = cls(name, api, skip_load=True)
        for key, value in six.iteritems(kwargs):
            setattr(obj, key, value)
        api.api_request("POST", cls.url, data=obj)
        return obj

    def save(self, api=None):
        """Save this object to the server. If the object does not exist it
        will be created.
        """
        api = api or self.api
        try:
            api.api_request("PUT", self.url, data=self)
        except ChefServerNotFoundError as e:
            # If you get a 404 during a save, just create it instead
            # This mirrors the logic in the Chef code
            api.api_request("POST", self.__class__.url, data=self)

    def delete(self, api=None):
        """Delete this object from the server."""
        api = api or self.api
        api.api_request("DELETE", self.url)

    def to_dict(self):
        d = {
            "name": self.name,
            "json_class": "Chef::" + self.__class__.__name__,
            "chef_type": self.__class__.__name__.lower(),
        }
        for attr in six.iterkeys(self.__class__.attributes):
            d[attr] = getattr(self, attr)
        return d

    def __str__(self):
        return self.name

    def __repr__(self):
        return "<%s %s>" % (type(self).__name__, self)

    @classmethod
    def _check_api_version(cls, api):
        # Don't enforce anything if api is None, since there is sometimes a
        # use for creating Chef objects without an API connection (just for
        # serialization perhaps).
        if api and cls.api_version_parsed > api.version_parsed:
            raise ChefAPIVersionError(
                "Class %s is not compatible with API version %s"
                % (cls.__name__, api.version)
            )

    def get_acl(self):
        return Acl(self.__class__.url.strip("/"), self.name, self.api)



from unittest import TestCase, skip

from chef import Node
from chef.exceptions import ChefError
from chef.node import NodeAttributes
from chef.tests import ChefTestCase


class NodeAttributeTestCase(TestCase):
    def test_getitem(self):
        attrs = NodeAttributes([{"a": 1}])
        self.assertEqual(attrs["a"], 1)

    def test_setitem(self):
        data = {"a": 1}
        attrs = NodeAttributes([data], write=data)
        attrs["a"] = 2
        self.assertEqual(attrs["a"], 2)
        self.assertEqual(data["a"], 2)

    def test_getitem_nested(self):
        attrs = NodeAttributes([{"a": {"b": 1}}])
        self.assertEqual(attrs["a"]["b"], 1)

    def test_set_nested(self):
        data = {"a": {"b": 1}}
        attrs = NodeAttributes([data], write=data)
        attrs["a"]["b"] = 2
        self.assertEqual(attrs["a"]["b"], 2)
        self.assertEqual(data["a"]["b"], 2)

    def test_search_path(self):
        attrs = NodeAttributes([{"a": 1}, {"a": 2}])
        self.assertEqual(attrs["a"], 1)

    def test_search_path_nested(self):
        data1 = {"a": {"b": 1}}
        data2 = {"a": {"b": 2}}
        attrs = NodeAttributes([data1, data2])
        self.assertEqual(attrs["a"]["b"], 1)

    def test_read_only(self):
        attrs = NodeAttributes([{"a": 1}])
        with self.assertRaises(ChefError):
            attrs["a"] = 2

    def test_get(self):
        attrs = NodeAttributes([{"a": 1}])
        self.assertEqual(attrs.get("a"), 1)

    def test_get_default(self):
        attrs = NodeAttributes([{"a": 1}])
        self.assertEqual(attrs.get("b"), None)

    def test_getitem_keyerror(self):
        attrs = NodeAttributes([{"a": 1}])
        with self.assertRaises(KeyError):
            attrs["b"]

    def test_iter(self):
        attrs = NodeAttributes([{"a": 1, "b": 2}])
        self.assertEqual(set(attrs), set(["a", "b"]))

    def test_iter2(self):
        attrs = NodeAttributes([{"a": {"b": 1, "c": 2}}])
        self.assertEqual(set(attrs["a"]), set(["b", "c"]))

    def test_len(self):
        attrs = NodeAttributes([{"a": 1, "b": 2}])
        self.assertEqual(len(attrs), 2)

    def test_len2(self):
        attrs = NodeAttributes([{"a": {"b": 1, "c": 2}}])
        self.assertEqual(len(attrs), 1)
        self.assertEqual(len(attrs["a"]), 2)

    def test_get_dotted(self):
        attrs = NodeAttributes([{"a": {"b": 1}}])
        self.assertEqual(attrs.get_dotted("a.b"), 1)

    def test_get_dotted_keyerror(self):
        attrs = NodeAttributes([{"a": {"b": 1}}])
        with self.assertRaises(KeyError):
            attrs.get_dotted("a.b.c")

    def test_set_dotted(self):
        data = {"a": {"b": 1}}
        attrs = NodeAttributes([data], write=data)
        attrs.set_dotted("a.b", 2)
        self.assertEqual(attrs["a"]["b"], 2)
        self.assertEqual(attrs.get_dotted("a.b"), 2)
        self.assertEqual(data["a"]["b"], 2)

    def test_set_dotted2(self):
        data = {"a": {"b": 1}}
        attrs = NodeAttributes([data], write=data)
        attrs.set_dotted("a.c.d", 2)
        self.assertEqual(attrs["a"]["c"]["d"], 2)
        self.assertEqual(attrs.get_dotted("a.c.d"), 2)
        self.assertEqual(data["a"]["c"]["d"], 2)


class NodeTestCase(ChefTestCase):
    def setUp(self):
        super(NodeTestCase, self).setUp()
        self.node = Node("test_1")

    def test_default_attr(self):
        self.assertEqual(self.node.default["test_attr"], "default")

    def test_normal_attr(self):
        self.assertEqual(self.node.normal["test_attr"], "normal")

    def test_override_attr(self):
        self.assertEqual(self.node.override["test_attr"], "override")

    def test_composite_attr(self):
        self.assertEqual(self.node.attributes["test_attr"], "override")

    def test_getitem(self):
        self.assertEqual(self.node["test_attr"], "override")

    def test_create(self):
        name = self.random()
        node = Node.create(name, run_list=["recipe[foo]"])
        self.register(node)
        self.assertEqual(node.run_list, ["recipe[foo]"])

        node2 = Node(name)
        self.assertTrue(node2.exists)
        self.assertEqual(node2.run_list, ["recipe[foo]"])

    def test_create_crosslink(self):
        node = Node.create(self.random())
        self.register(node)
        node.normal["foo"] = "bar"
        self.assertEqual(node["foo"], "bar")
        node.attributes["foo"] = "baz"
        self.assertEqual(node.normal["foo"], "baz")



from chef.base import ChefObject


class Role(ChefObject):
    """A Chef role object."""

    url = "/roles"
    attributes = {
        "description": str,
        "run_list": list,
        "default_attributes": dict,
        "override_attributes": dict,
        "env_run_lists": dict,
    }



class Permissions(object):
    """Represents single permissions object with list of actors and groups"""

    def __init__(self):
        self.actors = []
        self.groups = []

    def to_dict(self):
        d = {"actors": self.actors, "groups": self.groups}

        return d



import pytest
import json
import os
from unittest import mock
from chef.api import autoconfigure_v2
from chef.tests import TEST_ROOT


@pytest.fixture
def mock_client_key():
    return os.path.join(TEST_ROOT, "client.pem")


@pytest.fixture
def mock_client_config(mock_client_key):
    """
    Example load_config_json output
    """

    return json.dumps(
        {
            "log_location": "",
            "config_file": "/etc/chef/client.rb",
            "log_level": "warn",
            "node_name": "example-hostname",
            "client_key": "{client_key}".format(client_key=mock_client_key),
            "chef_server_url": "https://chef.example.com/organizations/example/",
            "syntax_check_cache_path": "",
            "cookbook_path": [
                "/home/username/src/example.com/chef/cookbooks",
                "/home/username/src/example.com/chef/site-cookbooks",
            ],
            "color": True,
            "script_path": [
                "/home/username/.chef/scripts",
                "/home/username/.chef/scripts",
            ],
            "file_backup_path": "/var/lib/chef",
            "file_cache_path": "/var/cache/chef",
            "pid_file": "/var/run/chef/client.pid",
            "ssl_verify_mode": "verify_peer",
            "automatic_attribute_blacklist": [
                "filesystem",
                "filesystem2",
                "counters",
                "sysconf",
                "etc/group",
                "pci",
                "block_device",
                "sessions",
            ],
            "rubygems_url": "https://rubygems.org/",
            "force_logger": False,
            "force_formatter": False,
            "profile_ruby": False,
        }
    )


@mock.patch("subprocess.check_call")
def test_autoconfigure_v2_ssl_verify_peer(mock_proc, mock_client_config):
    mock_proc.return_value = mock_client_config
    client = autoconfigure_v2()

    assert client.url == "https://chef.example.com/organizations/example"
    assert client.ssl_verify == True
    assert client.client == "example-hostname"


@mock.patch("subprocess.check_call")
def test_autoconfigure_v2_malformed(mock_proc):
    mock_proc.return_value = "No such file or directory"

    with pytest.raises(ValueError):
        autoconfigure_v2()



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
import os

from setuptools import setup, find_packages
from chef import __version__
setup(
    name = 'Py310Chef',
    version = __version__,
    packages = find_packages(),
    author = 'Noah Kantrowitz',
    author_email = 'noah@coderanger.net',
    maintainer = 'Tom Dierckx',
    maintainer_email = 'tom.dierckx@digipolis.be',
    description = '(upgraded) Python implementation of a Chef API client.',
    long_description = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    license = 'Apache 2.0',
    keywords = '',
    url = 'http://github.com/tomdierckx/pychef',
    classifiers = [],
    zip_safe = False,
    install_requires = ['six>=1.9.0','requests>=2.7.0'],
    test_suite = 'pytest',
    python_requires='>3.6.9',
    include_package_data=True
)



# Copyright (c) 2010 Noah Kantrowitz <noah@coderanger.net>

__version__ = "0.4.0rc3"

from chef.api import ChefAPI, autoconfigure
from chef.client import Client
from chef.data_bag import DataBag, DataBagItem
from chef.exceptions import ChefError
from chef.node import Node
from chef.role import Role
from chef.environment import Environment
from chef.search import Search
from chef.acl import Acl
